@@ -800,7 +800,13 @@ struct _NMSettInfoProperty {
                                const NMSettInfoProperty *property_info,
                                NMSetting                *setting,
                                const char               *src);
-    } direct_set_fcn;
+
+        /* We implement %NM_VALUE_TYPE_ENUM properties as integer GObject properties
+        * because using real enum triggers glib assertions when passing newer values to
+        * clients with old libnm. This defines the enum type that the direct_property of
+        * type %NM_VALUE_TYPE_ENUM will use. */
+        GType enum_gtype;
+    } direct_data;
 
     /* For direct properties, this is the param_spec that also should be
      * notified on changes. */
@@ -815,6 +821,10 @@ struct _NMSettInfoProperty {
      * normalize the string via g_ascii_strdown(). */
     bool direct_set_string_ascii_strdown : 1;
 
+    /* If TRUE, this is a NM_VALUE_TYPE_STRV direct property holding MAC addresses,
+     * and the setter will normalize them via _nm_utils_hwaddr_canonical_or_invalid(). */
+    bool direct_set_strv_normalize_hwaddr : 1;
+
     /* If TRUE, this is a NM_VALUE_TYPE_STRING direct property, and the setter will
      * normalize the string via g_strstrip(). */
     bool direct_set_string_strip : 1;
@@ -864,6 +874,10 @@ struct _NMSettInfoProperty {
      * an empty array. */
     bool direct_strv_preserve_empty : 1;
 
+    /* This flag indicates that an empty strv array should be returned
+     * instead of NULL if it hadn't been created yet. */
+    bool direct_strv_not_null : 1;
+
     /* Usually, properties that are set to the default value for the GParamSpec
      * are not serialized to GVariant (and NULL is returned by to_dbus_data().
      * Set this flag to force always converting the property even if the value
@@ -907,6 +921,14 @@ struct _NMSettInfoProperty {
      * is not deprecated. This flag is about the deprecation of the D-Bus representation
      * of a property. */
     bool dbus_deprecated : 1;
+
+    /* Whether the property is an alias.
+     *
+     * This flag indicates whether a property is an alias of another. This is used
+     * during _init_direct() to allow two or more properties to set the value of
+     * the same field.
+     */
+    bool direct_is_aliased_field : 1;
 };
 
 typedef struct {
